@@ -4,7 +4,7 @@
 modname = 'vcreview'
 distname = 'cubicweb-vcreview'
 
-numversion = (0, 10, 1)
+numversion = (1, 0, 0)
 version = '.'.join(str(num) for num in numversion)
 
 license = 'LGPL'
@@ -19,8 +19,8 @@ classifiers = [
            'Programming Language :: JavaScript',
     ]
 
-__depends__ =  {'cubicweb': '>= 3.14.0',
-                'cubicweb-vcsfile': '>= 1.7.1',
+__depends__ =  {'cubicweb': '>= 3.15.8',
+                'cubicweb-vcsfile': '>= 1.12.0',
                 'cubicweb-comment': '>= 1.8.0',
                 'cubicweb-task': None,
                 'cubicweb-nosylist': None,
@@ -28,7 +28,7 @@ __depends__ =  {'cubicweb': '>= 3.14.0',
 
 
 from os import listdir as _listdir
-from os.path import join, isdir, exists
+from os.path import join, isdir
 from glob import glob
 
 THIS_CUBE_DIR = join('share', 'cubicweb', 'cubes', modname)
