@@ -44,7 +44,10 @@ class Diff2HTMLTransform(Transform):
         w(u'<div class="diffComment">\n')
         self._done_cb = self.stop_comment
         self._continue_cb = self.comment_line
-        for line in trdata.decode().splitlines():
+        lines = trdata.decode().split('\n')
+        if lines[-1] == '':
+            del lines[-1]
+        for line in lines:
             try:
                 cb = cbs[line.split(' ', 1)[0]]
             except KeyError:
