@@ -24,9 +24,9 @@ from cubicweb.view import EntityAdapter
 from cubicweb.selectors import is_instance
 
 from cubes.comment import entities as comment
-from cubes.vcsfile import entities as vcsfile
 from cubes.task import entities as task
 
+from cubes.vcreview import users_by_author
 
 class InsertionPoint(AnyEntity):
     __regid__ = 'InsertionPoint'
@@ -34,6 +34,13 @@ class InsertionPoint(AnyEntity):
     def parent(self):
         return self.point_of[0]
 
+_TIP_MOST_REVS = ('Any TIP, H '
+                  'ORDERBY H ASC, TIP DESC'
+                  'WHERE P eid %(p)s, '
+                  '      P patch_revision TIP, '
+                  '      TIP hidden H, '
+                  '      NOT EXISTS(R obsoletes TIP, '
+                  '                 P patch_revision R)')
 
 class Patch(AnyEntity):
     __regid__ = 'Patch'
@@ -43,7 +50,8 @@ class Patch(AnyEntity):
 
     @property
     def repository(self):
-        return self.patch_repository[0]
+        return self.patch_revision[0].from_repository[0]
+
 
     @property
     def revisions(self):
@@ -53,37 +61,17 @@ class Patch(AnyEntity):
         return set(vc.file.path for vc in self.patch_revision)
 
     def tip(self):
-        rset = self._cw.execute(
-            'Any VC,R,RA,RB,RC,RD ORDERBY RC DESC LIMIT 1 '
-            'WHERE X eid %(x)s, X patch_revision VC, VC from_revision R,'
-            'R author RA, R branch RB, R creation_date RC, R description RD',
-            {'x': self.eid})
-        if rset:
-            return rset.get_entity(0, 0)
-        return None
+        return self._cw.entity_from_eid(self.all_tips()[0][0])
 
-    non_final_states = set( ('deleted', 'reviewed', 'pending-review', 'in-progress') )
-    active_states = set( ('reviewed', 'pending-review', 'in-progress') )
+    def all_tips(self):
+        return list(self._cw.execute(_TIP_MOST_REVS, {'p': self.eid}))
 
-    def is_active(self):
-        return self.cw_adapt_to('IWorkflowable').state in self.active_states
+    final_states = set( ('folded', 'applied', 'rejected') )
 
+    @property
+    def is_final(self):
+        return self.cw_adapt_to('IWorkflowable').state in self.final_states
 
-@monkeypatch(vcsfile.DeletedVersionContent, 'patch')
-@property
-def patch(self):
-    return self.reverse_patch_revision and self.reverse_patch_revision[0]
-
-
-@monkeypatch(vcsfile.Repository, 'patchrepo_of')
-@property
-def patchrepo_of(self):
-    return self.reverse_has_patch_repository and self.reverse_has_patch_repository[0]
-
-@monkeypatch(vcsfile.Repository, 'patchrepo')
-@property
-def patchrepo(self):
-    return self.has_patch_repository and self.has_patch_repository[0]
 
 @monkeypatch(task.Task, 'activity_of')
 @property
@@ -107,14 +95,23 @@ class PatchReviewBehaviour(EntityAdapter):
     def reviewers_rset(self):
         # ensure patch author can't review his own patch
         return self._cw.execute(
-            'Any U WHERE EXISTS(U in_group G, G name "reviewers") '
-            'OR EXISTS(P patch_repository R, R repository_reviewer U), '
-            'NOT P created_by U, P eid %(p)s',
+            'DISTINCT Any U '
+            'WHERE P eid %(p)s, '
+            '      P patch_revision RE, '
+            '      NOT P created_by U, '
+            '      EXISTS(U in_group G, G name "reviewers") '
+            '      OR EXISTS(RE from_repository R, '
+            '                R repository_reviewer U)',
             {'p': self.entity.eid})
 
-    def set_reviewers(self, transition):
+    def set_reviewers(self):
         if self.entity.patch_reviewer:
             return
+        # look for patch authors, so we don't ask them to review their own patch
+        authors = set()
+        for rev in self.entity.patch_revision:
+            authors.update(users_by_author(self._cw.execute, rev.author))
+
         # XXX only consider review affected during the last week by getting date
         # of the first in-progress -> pending-review transition? or use
         # something like a review_started_on attribute on patches ?
@@ -124,6 +121,8 @@ class PatchReviewBehaviour(EntityAdapter):
         ueids = []
         mincount = None
         for ueid, in self.reviewers_rset():
+            if ueid in authors:
+                continue
             count = nbpatches.get(ueid, 0)
             if mincount is None:
                 mincount = count
