@@ -18,24 +18,32 @@
 __docformat__ = "restructuredtext en"
 _ = unicode
 
-from yams.buildobjs import EntityType, RelationDefinition, String, Int
+from yams.buildobjs import (EntityType,
+                            RelationDefinition,
+                            String,
+                            Int,
+                            Boolean,
+                            SubjectRelation)
 
-from cubicweb.schema import WorkflowableEntityType, RQLConstraint, RRQLExpression
+from cubicweb.schema import (WorkflowableEntityType,
+                             RQLConstraint,
+                             RRQLExpression)
 
-# XXX nosylist. notifications: send all review (eg task + commment) at wf transition time
+from cubes.vcsfile.schema import Repository
 
 
-class has_patch_repository(RelationDefinition):
-    """the patch repository will have patch entities generated for file stored
-    in it.
-    """
-    subject = 'Repository'
-    object = 'Repository'
-    cardinality = '*?'
-
+Repository.add_relation(Boolean(default=False), name='has_review')
 
 # the patch entity #############################################################
 
+
+_SAME_REPO_CONSTRAINT = '''NOT EXISTS(
+    S patch_revision RE,
+    RE from_repository RO,
+    O from_repository R,
+    NOT R identity RO
+)'''
+
 class Patch(WorkflowableEntityType):
     __permissions__ = {
         'read': ('managers', 'users', 'guests'),
@@ -43,53 +51,21 @@ class Patch(WorkflowableEntityType):
         'update': ('managers', 'users'),
         'delete': ('managers',),
         }
-    branch = String(
-        maxsize=100, indexed=True,
-        description=_('branch where this patch has been introduced'))
-    originator = String(
-        maxsize=256, indexed=True,
-        description=_('author of the revision which has introduced this patch'))
     patch_name = String(
         required=True,
         description=_('name of the patch in the repository'))
+    patch_revision = SubjectRelation('Revision',
+                                     __permissions__={
+                                         'read': ('managers', 'users', 'guests'),
+                                         'add': (), # created by an internal session in a looping task
+                                         'delete': (),
+                                         },
+                                     constraints=[RQLConstraint(_SAME_REPO_CONSTRAINT)],
+                                     cardinality='+?')
 
 
-class patch_repository(RelationDefinition):
-    # shortcut for P patch_revision VC, VC revision REV, REV from_repository R
-    __permissions__ = {
-        'read': ('managers', 'users', 'guests'),
-        'add': (), # created by an internal session in a looping task
-        'delete': (),
-        }
-    subject = 'Patch'
-    object = 'Repository'
-    cardinality = '1*'
-    composite = 'object'
-    inlined = True
-
-class patch_revision(RelationDefinition):
-    __permissions__ = {
-        'read': ('managers', 'users', 'guests'),
-        'add': (), # created by an internal session in a looping task
-        'delete': (),
-        }
-    subject = 'Patch'
-    object = ('VersionContent', 'DeletedVersionContent')
-    cardinality = '+*'
-
-class applied_at(RelationDefinition):
-    __permissions__ = {
-        'read': ('managers', 'users', 'guests'),
-        'add': ('managers',),
-        'delete': ('managers',),
-        }
-    subject = 'Patch'
-    object = 'Revision'
-    cardinality = '??'
-    constraints = [RQLConstraint('S patch_repository PR, O from_repository R, '
-                                 'R has_patch_repository PR')]
-
 class repository_committer(RelationDefinition):
+    """integrator (review level 2)"""
     __permissions__ = {
         'read': ('managers', 'users', 'guests'),
         'add': ('managers',),
@@ -99,58 +75,56 @@ class repository_committer(RelationDefinition):
     object = 'CWUser'
 
 class repository_reviewer(RelationDefinition):
+    """(review level 1)"""
     __permissions__ = {
         'read': ('managers', 'users', 'guests'),
         'add': ('managers', 'committers',
-                RRQLExpression('S has_patch_repository R, R repository_committer U')),
+                RRQLExpression('S repository_committer U')),
         'delete': ('managers', 'committers',
-                   RRQLExpression('S has_patch_repository R, R repository_committer U')),
+                   RRQLExpression('S repository_committer U')),
         }
     subject = 'Repository'
     object = 'CWUser'
 
 class patch_reviewer(RelationDefinition):
+    """The good fellow reviewing a single patch"""
     __permissions__ = {
         'read': ('managers', 'users', 'guests'),
         'add': ('managers', 'committers',
-                RRQLExpression('S patch_repository R, R repository_committer U')),
+                RRQLExpression('S repository_committer U')),
         'delete': ('managers', 'committers',
-                   RRQLExpression('S patch_repository R, R repository_committer U')),
+                   RRQLExpression('S repository_committer U')),
         }
     subject = 'Patch'
     object = 'CWUser'
     constraints = [RQLConstraint('EXISTS(O in_group G, G name "reviewers") '
-                                 'OR EXISTS(S patch_repository R, R repository_reviewer O)')]
+                                 'OR EXISTS(S patch_revision RE, '
+                                 '          RE from_repository R, '
+                                 '          R repository_reviewer O)')]
 
 class patch_committer(RelationDefinition):
+    """The level 2 guy in charge of getting the patch published"""
     __permissions__ = {
         'read': ('managers', 'users', 'guests'),
         'add': ('managers', 'committers',
-                RRQLExpression('S patch_repository R, R repository_committer U')),
+                RRQLExpression('S repository_committer U')),
         'delete': ('managers', 'committers',
-                   RRQLExpression('S patch_repository R, R repository_committer U')),
+                   RRQLExpression('S repository_committer U')),
         }
     subject = 'Patch'
     object = 'CWUser'
     constraints = [RQLConstraint('EXISTS(O in_group G, G name "committers") '
-                                 'OR EXISTS(S patch_repository R, R repository_committer O)')]
-
-class patch_depends_on(RelationDefinition):
-    subject = 'Patch'
-    object = 'Patch'
+                                 'OR EXISTS(S patch_revision RE, RE from_repository R, R repository_committer O)')]
 
 # ability to reference diff chunks #############################################
 
 class InsertionPoint(EntityType):
+    """diff hunk"""
     __unique_together__ = [('lid', 'point_of')]
     lid = Int(indexed=True, required=True)
 
-class point_of(RelationDefinition):
-    subject = 'InsertionPoint'
-    object = 'VersionContent'
-    cardinality = '1*'
-    composite = 'object'
-    inlined = True
+    point_of = SubjectRelation('Revision', cardinality = '1*',
+                               composite = 'object', inlined = True)
 
 
 # activities ###################################################################
@@ -174,9 +148,7 @@ class comments(RelationDefinition):
 # XXX Comment should be in list, but this will conflict with the forge cube when
 # using vcreview
 class nosy_list(RelationDefinition):
-    subject = ('Repository', 'Patch', 'Task',
-               'VersionContent', 'DeletedVersionContent',
-               'InsertionPoint')
+    subject = ('Repository', 'Patch', 'Revision', 'Task', 'InsertionPoint')
     object = 'CWUser'
 
 class interested_in(RelationDefinition):
