@@ -22,6 +22,13 @@ COMPONENT_CONTEXT = 'navcontentbottom'
 def insert_point_callback(ipid, trdata, w):
     context = trdata.appobject
     cw = context._cw
+    e_schema = getattr(context, 'e_schema', None)
+    if e_schema is None:
+        # not an entity
+        return
+    if 'point_of' not in e_schema.objrels:
+        # no InsertionPoint on this entity
+        return
     ipoint = cw.execute('InsertionPoint IP WHERE IP point_of X, X eid %(x)s, IP lid %(id)s',
                         {'x': context.eid, 'id': ipid})
     if ipoint:
